@@ -71,6 +71,33 @@ class TreeNode:
     children = attr.ib(factory=list)
     label = attr.ib(default=None)
 
+    def as_tables(self, *, num_leaves, span, branch_length):
+        """
+        Convert the tree rooted at this node into an equivalent
+        TableCollection. Internal nodes are allocated in postorder.
+        """
+        tables = tskit.TableCollection(span)
+        for _ in range(num_leaves):
+            tables.nodes.add_row(flags=tskit.NODE_IS_SAMPLE, time=0)
+
+        def assign_internal_labels(node):
+            if len(node.children) == 0:
+                node.time = 0
+            else:
+                max_child_time = 0
+                for child in node.children:
+                    assign_internal_labels(child)
+                    max_child_time = max(max_child_time, child.time)
+                node.time = max_child_time + branch_length
+                node.label = tables.nodes.add_row(time=node.time)
+                for child in node.children:
+                    tables.edges.add_row(0, span, node.label, child.label)
+
+        # Do a postorder traversal to assign the internal node labels and times.
+        assign_internal_labels(self)
+        tables.sort()
+        return tables
+
     @staticmethod
     def random_binary_tree(leaf_labels, rng):
         """
@@ -104,6 +131,24 @@ class TreeNode:
         root = nodes[0]
         while root.parent is not None:
             root = root.parent
+
+        # Canonicalise the order of the children within a node. This
+        # is given by (num_leaves, min_label). See also the
+        # RankTree.canonical_order function for the definition of
+        # how these are ordered during rank/unrank.
+
+        def reorder_children(node):
+            if len(node.children) == 0:
+                return 1, node.label
+            keys = [reorder_children(child) for child in node.children]
+            if keys[0] > keys[1]:
+                node.children = node.children[::-1]
+            return (
+                sum(leaf_count for leaf_count, _ in keys),
+                min(min_label for _, min_label in keys),
+            )
+
+        reorder_children(root)
         return root
 
     @classmethod
@@ -127,7 +172,7 @@ class TreeNode:
         return root
 
 
-def generate_star(num_leaves, *, span, branch_length, record_provenance):
+def generate_star(num_leaves, *, span, branch_length, record_provenance, **kwargs):
     """
     Generate a star tree for the specified number of leaves.
 
@@ -155,10 +200,10 @@ def generate_star(num_leaves, *, span, branch_length, record_provenance):
         tables.provenances.add_row(
             record=json.dumps(tskit.provenance.get_provenance_dict(parameters))
         )
-    return tables.tree_sequence().first()
+    return tables.tree_sequence().first(**kwargs)
 
 
-def generate_comb(num_leaves, *, span, branch_length, record_provenance):
+def generate_comb(num_leaves, *, span, branch_length, record_provenance, **kwargs):
     """
     Generate a comb tree for the specified number of leaves.
 
@@ -188,10 +233,12 @@ def generate_comb(num_leaves, *, span, branch_length, record_provenance):
         tables.provenances.add_row(
             record=json.dumps(tskit.provenance.get_provenance_dict(parameters))
         )
-    return tables.tree_sequence().first()
+    return tables.tree_sequence().first(**kwargs)
 
 
-def generate_balanced(num_leaves, *, arity, span, branch_length, record_provenance):
+def generate_balanced(
+    num_leaves, *, arity, span, branch_length, record_provenance, **kwargs
+):
     """
     Generate a balanced tree for the specified number of leaves.
 
@@ -202,27 +249,10 @@ def generate_balanced(num_leaves, *, arity, span, branch_length, record_provenan
     if arity < 2:
         raise ValueError("The arity must be at least 2")
 
-    tables = tskit.TableCollection(span)
-    for _ in range(num_leaves):
-        tables.nodes.add_row(flags=tskit.NODE_IS_SAMPLE, time=0)
-
-    def assign_internal_labels(node):
-        if len(node.children) == 0:
-            node.time = 0
-        else:
-            max_child_time = 0
-            for child in node.children:
-                assign_internal_labels(child)
-                max_child_time = max(max_child_time, child.time)
-            node.time = max_child_time + branch_length
-            node.label = tables.nodes.add_row(time=node.time)
-            for child in node.children:
-                tables.edges.add_row(0, span, node.label, child.label)
-
     root = TreeNode.balanced_tree(range(num_leaves), arity)
-    # Do a postorder traversal to assign the internal node labels and times.
-    assign_internal_labels(root)
-    tables.sort()
+    tables = root.as_tables(
+        num_leaves=num_leaves, span=span, branch_length=branch_length
+    )
 
     if record_provenance:
         # TODO replace with a version of https://github.com/tskit-dev/tskit/pull/243
@@ -233,7 +263,36 @@ def generate_balanced(num_leaves, *, arity, span, branch_length, record_provenan
             record=json.dumps(tskit.provenance.get_provenance_dict(parameters))
         )
 
-    return tables.tree_sequence().first()
+    return tables.tree_sequence().first(**kwargs)
+
+
+def generate_random_binary(
+    num_leaves, *, span, branch_length, random_seed, record_provenance, **kwargs
+):
+    """
+    Sample a leaf-labelled binary tree uniformly.
+
+    See the documentation for :meth:`Tree.generate_random_binary` for more details.
+    """
+    if num_leaves < 1:
+        raise ValueError("The number of leaves must be at least 1")
+
+    rng = random.Random(random_seed)
+    root = TreeNode.random_binary_tree(range(num_leaves), rng)
+    tables = root.as_tables(
+        num_leaves=num_leaves, span=span, branch_length=branch_length
+    )
+
+    if record_provenance:
+        # TODO replace with a version of https://github.com/tskit-dev/tskit/pull/243
+        # TODO also make sure we convert all the arguments so that they are
+        # definitely JSON encodable.
+        parameters = {"command": "generate_random_binary", "TODO": "add parameters"}
+        tables.provenances.add_row(
+            record=json.dumps(tskit.provenance.get_provenance_dict(parameters))
+        )
+    ts = tables.tree_sequence()
+    return ts.first(**kwargs)
 
 
 def split_polytomies(
@@ -243,18 +302,16 @@ def split_polytomies(
     method=None,
     record_provenance=True,
     random_seed=None,
+    **kwargs,
 ):
     """
     Return a new tree where extra nodes and edges have been inserted
     so that any any node with more than two children is resolved into
     a binary tree.
 
-    For further documentation, please refer to the :meth:`Tree.split_polytomies`
-    method, which is the usual route through which this function is called.
+    See the documentation for :meth:`Tree.split_polytomies` for more details.
     """
     allowed_methods = ["random"]
-    if epsilon is None:
-        epsilon = 1e-10
     if method is None:
         method = "random"
     if method not in allowed_methods:
@@ -269,15 +326,57 @@ def split_polytomies(
         if tree.num_children(u) > 2:
             root = TreeNode.random_binary_tree(tree.children(u), rng)
             root.label = u
-            time = tree.time(u) - epsilon
-            stack = [(child, time) for child in root.children]
+            root_time = tree.time(u)
+            stack = [(child, root_time) for child in root.children]
             while len(stack) > 0:
-                node, time = stack.pop()
+                node, parent_time = stack.pop()
                 if node.label is None:
-                    node.label = tables.nodes.add_row(time=time)
+                    if epsilon is None:
+                        child_time = np.nextafter(parent_time, -np.inf)
+                    else:
+                        child_time = parent_time - epsilon
+                    node.label = tables.nodes.add_row(time=child_time)
+                else:
+                    assert len(node.children) == 0
+                    # This is a leaf node connecting back into the original tree
+                    child_time = tree.time(node.label)
+                if parent_time <= child_time:
+                    u = root.label
+                    min_child_time = min(tree.time(v) for v in tree.children(u))
+                    min_time = root_time - min_child_time
+                    message = (
+                        f"Cannot resolve the degree {tree.num_children(u)} "
+                        f"polytomy rooted at node {u} with minimum time difference "
+                        f"of {min_time} to the resolved leaves."
+                    )
+                    if epsilon is None:
+                        message += (
+                            " The time difference between nodes is so small that "
+                            "more nodes cannot be inserted between within the limits "
+                            "of floating point precision."
+                        )
+                    else:
+                        # We can also have parent_time == child_time if epsilon is
+                        # chosen such that we exactly divide up the branch in the
+                        # original tree. We avoid saying this is caused by a
+                        # too-small epsilon by noting it can only happen when we
+                        # are at leaf node in the randomly generated tree.
+                        if parent_time == child_time and len(node.children) > 0:
+                            message += (
+                                f" The fixed epsilon value of {epsilon} is too small, "
+                                "resulting in the parent and child times being equal "
+                                "within the limits of numerical precision."
+                            )
+                        else:
+                            message += (
+                                f" The fixed epsilon value of {epsilon} is too large, "
+                                "resulting in the parent time being less than the child "
+                                "time."
+                            )
+                    raise tskit.LibraryError(message)
                 tables.edges.add_row(*tree.interval, node.parent.label, node.label)
                 for child in node.children:
-                    stack.append((child, time - epsilon))
+                    stack.append((child, child_time))
         else:
             for v in tree.children(u):
                 tables.edges.add_row(*tree.interval, u, v)
@@ -292,21 +391,25 @@ def split_polytomies(
         ts = tables.tree_sequence()
     except tskit.LibraryError as e:
         msg = str(e)
+        # We should have caught all topology time travel above.
+        assert not msg.startswith("time[parent] must be greater than time[child]")
         if msg.startswith(
             "A mutation's time must be < the parent node of the edge on which it occurs"
         ):
-            e.args += (
-                f"Epsilon={epsilon} not small enough to create new nodes below a "
-                "polytomy, due to the time of a mutation above a child of the polytomy.",
-            )
-        elif msg.startswith("time[parent] must be greater than time[child]"):
-            e.args += (
-                f"Epsilon={epsilon} not small enough to create new nodes below a "
-                "polytomy, due to the time of one of child nodes being too close. ",
-            )
-
+            if epsilon is not None:
+                msg = (
+                    f"epsilon={epsilon} not small enough to create new nodes below a "
+                    "polytomy, due to the time of a mutation above a child of the "
+                    "polytomy."
+                )
+            else:
+                msg = (
+                    "Cannot split polytomy: mutation with numerical precision "
+                    "of the parent time."
+                )
+            e.args += (msg,)
         raise e
-    return ts.at(tree.interval[0])
+    return ts.at(tree.interval[0], **kwargs)
 
 
 def treeseq_count_topologies(ts, sample_sets):
